﻿using Un4seen.Bass;
using Un4seen.Bass.AddOn.Fx;

namespace WpfAudioPlayer.Models.Audio
{
	/// <summary>
	/// イコライザ機能を提供します。
	/// </summary>
	class Equalizer
	{
		/// <summary>
		/// ストリームにイコライザを割り当てます。
		/// </summary>
		/// <param name="stream">ストリームの識別子。</param>
		public void Attach( int stream )
		{
			this._equalizer = Bass.BASS_ChannelSetFX( stream, BASSFXType.BASS_FX_BFX_PEAKEQ, 0 );

			var eq = new BASS_BFX_PEAKEQ() { fBandwidth = Equalizer.BandWidth, fQ = 0, lChannel = BASSFXChan.BASS_BFX_CHANALL };
			for( int band = 0; band < this._gains.Length; ++band )
			{
				eq.fGain = this._gains[ band ];
				eq.lBand = band;
				eq.fCenter = Equalizer.GainCenters[ band ];

				Bass.BASS_FXSetParameters( this._equalizer, eq );
			}
		}

		/// <summary>
		/// イコライザを更新します。
		/// </summary>
		/// <param name="band">帯域。</param>
		/// <param name="gain">ゲイン。</param>
		private void Update( int band, float gain )
		{
			this._gains[ band ] = gain;
			var eq = new BASS_BFX_PEAKEQ() { lBand = band, fCenter = Equalizer.GainCenters[ band ], fBandwidth = Equalizer.BandWidth, fQ = 0, lChannel = BASSFXChan.BASS_BFX_CHANALL, fGain = gain };
			Bass.BASS_FXSetParameters( this._equalizer, eq );
		}

		/// <summary>
		/// 32 Hz のゲインを取得または設定します。
		/// </summary>
		public float Gain32
		{
			get
			{
				return this._gains[ 0 ];
			}
			set
			{
				this.Update( 0, value );
			}
		}

		/// <summary>
		/// 64 Hz のゲインを取得または設定します。
		/// </summary>
		public float Gain64
		{
			get
			{
				return this._gains[ 1 ];
			}
			set
			{
				this.Update( 1, value );
			}
		}

		/// <summary>
		/// 125 Hz のゲインを取得または設定します。
		/// </summary>
		public float Gain125
		{
			get
			{
				return this._gains[ 2 ];
			}
			set
			{
				this.Update( 2, value );
			}
		}

		/// <summary>
		/// 250 Hz のゲインを取得または設定します。
		/// </summary>
		public float Gain250
		{
			get
			{
				return this._gains[ 3 ];
			}
			set
			{
				this.Update( 3, value );
			}
		}

		/// <summary>
		/// 500 Hz のゲインを取得または設定します。
		/// </summary>
		public float Gain500
		{
			get
			{
				return this._gains[ 4 ];
			}
			set
			{
				this.Update( 4, value );
			}
		}

		/// <summary>
		/// 1 KHz のゲインを取得または設定します。
		/// </summary>
		public float Gain1K
		{
			get
			{
				return this._gains[ 5 ];
			}
			set
			{
				this.Update( 5, value );
			}
		}

		/// <summary>
		/// 2 KHz のゲインを取得または設定します。
		/// </summary>
		public float Gain2K
		{
			get
			{
				return this._gains[ 6 ];
			}
			set
			{
				this.Update( 6, value );
			}
		}

		/// <summary>
		/// 4 KHz のゲインを取得または設定します。
		/// </summary>
		public float Gain4K
		{
			get
			{
				return this._gains[ 7 ];
			}
			set
			{
				this.Update( 7, value );
			}
		}

		/// <summary>
		/// 8 KHz のゲインを取得または設定します。
		/// </summary>
		public float Gain8K
		{
			get
			{
				return this._gains[ 8 ];
			}
			set
			{
				this.Update( 8, value );
			}
		}

		/// <summary>
		/// 16 KHz のゲインを取得または設定します。
		/// </summary>
		public float Gain16K
		{
			get
			{
				return this._gains[ 9 ];
			}
			set
			{
				this.Update( 9, value );
			}
		}

		/// <summary>
		/// 帯域幅。
		/// </summary>
		private const float BandWidth = 0.5f;

		/// <summary>
		/// 各帯域の中心周波数のコレクション。
		/// </summary>
		private static readonly int[] GainCenters = new int[ 10 ] { 32, 64, 125, 250, 500, 1000, 2000, 4000, 8000, 16000 };

		/// <summary>
		/// Bass.Net が生成したイコライザのハンドル。
		/// </summary>
		private int _equalizer;

		/// <summary>
		/// 帯域毎のゲインのコレクション。
		/// </summary>
		private float[] _gains = new float[ 10 ] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	}
}
