﻿using System;
using System.Text;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;

namespace WpfAudioPlayer.Models.Wmf
{
	/// <summary>
	/// The IWMSyncReader interface provides the ability to read ASF files using synchronous calls.
	/// This is in contrast to many of the methods in IWMReader, which are called asynchronously.
	/// </summary>
	[ComImport, Guid( "9397F121-7705-4dc9-B049-98B698188414" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMSyncReader
	{
		/// <summary>
		/// The Open method opens a file for reading.
		/// Unlike IWMReader::Open, this method is a synchronous call. 
		/// </summary>
		/// <param name="fileName">Pointer to a wide-character null-terminated string containing the file name to open. This must be a valid file name with an ASF file extension or an MP3 file name.</param>
		void Open( [In, MarshalAs( UnmanagedType.LPWStr )] string fileName );

		/// <summary>
		/// The Close method removes a file from the synchronous reader.
		/// </summary>
		void Close();

		/// <summary>
		/// The SetRange method enables you to specify a start time and duration for playback by the synchronous reader.
		/// </summary>
		/// <param name="startTime">Offset into the file at which to start playback. This value is measured in 100-nanosecond units.</param>
		/// <param name="duration">Duration in 100-nanosecond units, or zero to continue playback to the end of the file.</param>
		void SetRange( [In] ulong startTime, [In] long duration );

		/// <summary>
		/// The SetRangeByFrame method configures the synchronous reader to read a portion of the file specified by a starting video frame number and a number of frames to read.
		/// </summary>
		/// <param name="streamNumber">Stream number.</param>
		/// <param name="frameNumber">Frame number at which to begin playback. The first frame in a file is number 1.</param>
		/// <param name="framesToRead">Count of frames to read. Pass 0 to continue playback to the end of the file.</param>
		void SetRangeByFrame( [In] ushort streamNumber, [In] ulong frameNumber, [In] long framesToRead );

		/// <summary>
		/// The GetNextSample method retrieves the next sample from the file.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number for which you would like a sample. If you pass zero, the next sample in the file is returned, regardless of stream number.</param>
		/// <param name="sample">Pointer to a buffer that receives the sample. Set to NULL to retrieve the sample time without getting the sample. If set to NULL, pcnsDuration and pdwFlags must both be set to NULL as well.</param>
		/// <param name="sampleTime">Pointer to a QWORD variable that receives the sample time in 100-nanosecond units.</param>
		/// <param name="duration">Pointer to QWORD variable that receives the duration of the sample in 100-nanosecond units.</param>
		/// <param name="flags">Pointer to a DWORD containing one or more of the following flags.</param>
		/// <param name="outputNumber">Pointer to a DWORD that receives the output number.</param>
		/// <param name="nextStreamNumber">Pointer to a WORD that receives the stream number.</param>
		void GetNextSample( [In] ushort streamNumber, [Out] out INSSBuffer sample, [Out] out ulong sampleTime, [Out] out ulong duration, [Out] out uint flags, [Out] out uint outputNumber, [Out] out ushort nextStreamNumber );

		/// <summary>
		/// The SetStreamsSelected method configures the samples to be delivered from a list of streams.
		/// Each stream can be set to deliver all samples, no samples, or only cleanpoint samples. 
		/// </summary>
		/// <param name="streamCount">Count of streams listed at pwStreamNumbers.</param>
		/// <param name="streamNumbers">Pointer to an array of WORD values containing the stream numbers.</param>
		/// <param name="selections">Pointer to an array of WMT_STREAM_SELECTION enumeration values. These values correspond with the stream numbers listed at pwStreamNumbers. Each value specifies the samples to deliver for the appropriate stream.</param>
		void SetStreamsSelected( [In] ushort streamCount, [In, MarshalAs( UnmanagedType.LPArray, SizeParamIndex = 0 )] ushort[] streamNumbers, [In, MarshalAs( UnmanagedType.LPArray, SizeParamIndex = 0 )] WMT_STREAM_SELECTION[] selections );

		/// <summary>
		/// The GetStreamSelected method retrieves a flag indicating whether a particular stream is currently selected.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number.</param>
		/// <param name="selection">Pointer to a variable that receives one member of the WMT_STREAM_SELECTION enumeration type on output. This value specifies the selection status for the specified stream.</param>
		void GetStreamSelected( [In]ushort streamNumber, [Out] out WMT_STREAM_SELECTION selection );

		/// <summary>
		/// The SetReadStreamSamples method specifies whether samples from a stream will be delivered compressed or uncompressed.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number.</param>
		/// <param name="compressed">Boolean value that is True if samples will be compressed.</param>
		void SetReadStreamSamples( [In] ushort streamNumber, [In, MarshalAs( UnmanagedType.Bool )] bool compressed );

		/// <summary>
		/// The GetReadStreamSamples method ascertains whether a stream is configured to deliver compressed samples.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number.</param>
		/// <param name="compressed">Pointer to a flag that receives the status of compressed delivery for the stream specified.</param>
		void GetReadStreamSamples( [In] ushort streamNumber, [Out, MarshalAs( UnmanagedType.Bool )] out bool compressed );

		/// <summary>
		/// The GetOutputSetting method retrieves a setting for a particular output by name.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number.</param>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the name of the setting for which you want the value. For a list of global constants representing setting names, see Output Settings.</param>
		/// <param name="type">Pointer to a variable that receives one value from the WMT_ATTR_DATATYPE enumeration type. The value received specifies the type of data in pValue.</param>
		/// <param name="value">Pointer to a byte buffer containing the value. Pass NULL to retrieve the length of the buffer required.</param>
		/// <param name="valueLength">On input, pointer to a variable containing the length of pValue. On output, the variable contains the number of bytes in pValue used.</param>
		void GetOutputSetting( [In] uint outputNumber, [In, MarshalAs( UnmanagedType.LPWStr )] string name, [Out] out WMT_ATTR_DATATYPE type, [Out] IntPtr value, [In, Out] ref uint valueLength );

		/// <summary>
		/// The SetOutputSetting method specifies a named setting for a particular output.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number.</param>
		/// <param name="name">Pointer to a null-terminated string containing the name of the setting. For a list of global constants representing setting names, see Output Settings.</param>
		/// <param name="type">Member of the WMT_ATTR_DATATYPE enumeration type. This value specifies the type of data in the buffer at pValue.</param>
		/// <param name="value">Pointer to a byte array containing the value of the setting. The type of data stored in this buffer is specified by Type.</param>
		/// <param name="valueLength">Size of pValue in bytes.</param>
		void SetOutputSetting( [In] uint outputNumber, [In, MarshalAs( UnmanagedType.LPWStr )] string name, [In] WMT_ATTR_DATATYPE type, [In, MarshalAs( UnmanagedType.LPArray, SizeParamIndex = 4 )] byte[] value, [In] uint valueLength );

		/// <summary>
		/// The GetOutputCount method retrieves the number of outputs that exist for the file open in the synchronous reader.
		/// </summary>
		/// <param name="count">Pointer to a DWORD that receives the number of outputs in the file.</param>
		void GetOutputCount( [Out] out uint count );

		/// <summary>
		/// The GetOutputProps method retrieves the current properties of an uncompressed output stream.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number.</param>
		/// <param name="properties">Pointer to a pointer to an IWMOutputMediaProps interface, which is created by a successful call to this method.</param>
		void GetOutputProps( [In] uint outputNumber, [Out, MarshalAs( UnmanagedType.Interface )] out IWMOutputMediaProps properties );

		/// <summary>
		/// The SetOutputProps method specifies the media properties of an uncompressed output stream.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number.</param>
		/// <param name="properties">Pointer to an IWMOutputMediaProps interface.</param>
		void SetOutputProps( [In] uint outputNumber, [In, MarshalAs( UnmanagedType.Interface )] IWMOutputMediaProps properties );

		/// <summary>
		/// The GetOutputFormatCount method is used to determine all possible format types supported by this output on the synchronous reader.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number for which you want to determine the number of supported formats.</param>
		/// <param name="count">Pointer to a DWORD that receives the number of supported formats.</param>
		void GetOutputFormatCount( [In] uint outputNumber, [Out] out uint count );

		/// <summary>
		/// The GetOutputFormat method retrieves the supported formats for a specified output media stream.
		/// </summary>
		/// <param name="outputNumber">DWORD containing the output number.</param>
		/// <param name="formatNumber">DWORD containing the format number.</param>
		/// <param name="properties">Pointer to a pointer to an IWMOutputMediaProps interface. This object is created by a successful call to this method.</param>
		void GetOutputFormat( [In] uint outputNumber, [In] uint formatNumber, [Out, MarshalAs( UnmanagedType.Interface )] out IWMOutputMediaProps properties );

		/// <summary>
		/// The GetOutputNumberForStream method retrieves the output number that corresponds with the specified stream.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number for which you want to retrieve the corresponding output number.</param>
		/// <param name="outputNumber">Pointer to a DWORD that will receive the output number that corresponds to the stream number specified in wStreamNum.</param>
		void GetOutputNumberForStream( [In] ushort streamNumber, [Out] out uint outputNumber );

		/// <summary>
		/// The GetStreamNumberForOutput method retrieves the stream number that corresponds with the specified output.
		/// </summary>
		/// <param name="outputNumber">DWORD value specifying the output number for which you want to retrieve a stream number.</param>
		/// <param name="streamNumber">Pointer to a WORD value that receives the stream number that corresponds to the output specified by dwOutput.</param>
		void GetStreamNumberForOutput( [In] uint outputNumber, [Out] out ushort streamNumber );

		/// <summary>
		/// The GetMaxStreamSampleSize method retrieves the maximum sample size for a specified stream in the file that is open in the synchronous reader.
		/// </summary>
		/// <param name="output">WORD containing the stream number for which you want to retrieve the maximum sample size.</param>
		/// <param name="size">Pointer to a DWORD value that receives the maximum sample size, in bytes, for the stream specified in wStream.</param>
		void GetMaxOutputSampleSize( [In] uint output, [Out] out uint size );

		/// <summary>
		/// The GetMaxOutputSampleSize method retrieves the maximum sample size for a specified output of the file open in the synchronous reader.
		/// </summary>
		/// <param name="stream">DWORD containing the output number for which you want to retrieve the maximum sample size.</param>
		/// <param name="size">Pointer to a DWORD value that receives the maximum sample size, in bytes, for the output specified in dwOutput.</param>
		void GetMaxStreamSampleSize( [In] ushort stream, [Out] out uint size );

		/// <summary>
		/// The OpenStream method opens a stream for reading.
		/// </summary>
		/// <param name="stream">Pointer to an IStream interface.</param>
		void OpenStream( [In, MarshalAs( UnmanagedType.Interface )] IStream stream );
	}

	/// <summary>
	/// The IWMMediaProps interface sets and retrieves the WM_MEDIA_TYPE structure for an input, stream, or output.
	/// </summary>
	[ComImport, Guid( "96406BCE-2B2B-11d3-B36B-00C04F6108FF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMMediaProps
	{
		/// <summary>
		/// The GetType method retrieves the major type of the media in the stream, input, or output described by the object to which the current IWMMediaProps interface belongs.
		/// </summary>
		/// <param name="id">Pointer to a GUID specifying the media type.</param>
		void GetType( [Out] out Guid id );

		/// <summary>
		/// The GetMediaType method retrieves a structure describing the media type.
		/// </summary>
		/// <param name="type">Pointer to a WM_MEDIA_TYPE structure. If this parameter is set to NULL, this method returns the size of the buffer required in the pcbType parameter.</param>
		/// <param name="typeSize">On input, the size of the pType buffer. On output, if pType is set to NULL, the value this points to is set to the size of the buffer needed to hold the media type structure.</param>
		void GetMediaType( IntPtr type, [In, Out] ref uint typeSize );

		/// <summary>
		/// The SetMediaType method specifies the media type.
		/// </summary>
		/// <param name="type">Pointer to the WM_MEDIA_TYPE structure describing the input, stream, or output.</param>
		void SetMediaType( [In] ref WM_MEDIA_TYPE type );
	}

	/// <summary>
	/// The IWMOutputMediaProps interface is used to retrieve the properties of an output stream.
	/// </summary>
	[ComImport, Guid( "96406BD7-2B2B-11d3-B36B-00C04F6108FF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMOutputMediaProps : IWMMediaProps
	{
		/// <summary>
		/// The GetStreamGroupName method is not implemented in this release, and returns the empty string.
		/// </summary>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the name. Pass NULL to retrieve the length of the name.</param>
		/// <param name="length">On input, a pointer to a variable containing the length of the pwszName array in wide characters (2 bytes). On output, and if the method succeeds, the variable contains the length of the name, including the terminating null character.</param>
		void GetStreamGroupName( [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder name, [In, Out] ref ushort length );

		/// <summary>
		/// The GetConnectionName method retrieves the name of the connection to be used for output.
		/// </summary>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the name. Pass NULL to retrieve the length of the name.</param>
		/// <param name="length">On input, a pointer to a variable containing the length of the pwszName array in wide characters. On output, if the method succeeds, it specifies a pointer to the length of the connection name, including the terminating null character.</param>
		void GetConnectionName( [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder name, [In, Out] ref ushort length );
	}

	/// <summary>
	/// The INSSBuffer interface is the basic interface of a buffer object.
	/// A buffer object is a wrapper around a memory buffer.
	/// The methods exposed by this interface are used to manipulate the buffer.
	/// </summary>
	[ComImport, Guid( "E1CD3524-03D7-11d2-9EED-006097D2D7CF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface INSSBuffer
	{
		/// <summary>
		/// The GetLength method retrieves the size of the used portion of the buffer controlled by the buffer object.
		/// </summary>
		/// <param name="length">Pointer to a DWORD containing the length of the buffer, in bytes.</param>
		void GetLength( [Out] out uint length );

		/// <summary>
		/// The SetLength method specifies the size of the used portion of the buffer.
		/// If you are storing a sample in the buffer, call INSSBuffer::GetBuffer to retrieve the address of the buffer.
		/// Then copy your data to that address and use this method to set the length of the used portion of the buffer.
		/// </summary>
		/// <param name="length">DWORD containing the size of the used portion, in bytes.</param>
		void SetLength( [In] uint length );

		/// <summary>
		/// The GetMaxLength method retrieves the maximum size to which a buffer can be set.
		/// The maximum value is set when the sample is created.
		/// If you are using IWMWriter::AllocateSample, the size you specify becomes the maximum buffer length.
		/// The actual amount of the buffer that is used can be retrieved by calling INSSBuffer::GetLength.
		/// </summary>
		/// <param name="length">Pointer to a DWORD containing the maximum length, in bytes.</param>
		void GetMaxLength( [Out] out uint length );

		/// <summary>
		/// The GetBuffer method retrieves the location of the buffer controlled by the buffer object.
		/// Buffers are used to store samples. When passing samples to the writer, you need the location of the buffer so you can copy your samples into it.
		/// When you copy data to the address returned by this call, you must call INSSBuffer::SetLength to specify how much of the buffer actually contains data.
		/// </summary>
		/// <param name="buffer">Pointer to a pointer to the buffer.</param>
		void GetBuffer( [Out] out IntPtr buffer );

		/// <summary>
		/// The GetBufferAndLength method retrieves the location and size of the used portion of the buffer controlled by the buffer object.
		/// Buffers are used to store samples.
		/// When reading ASF files, you can use the location and length to copy a sample from a buffer delivered by the reader or synchronous reader.
		/// </summary>
		/// <param name="buffer">Pointer to a pointer to the buffer.</param>
		/// <param name="length">Pointer to a DWORD containing the length of the buffer, in bytes.</param>
		void GetBufferAndLength( [Out] out IntPtr buffer, [Out] out uint length );
	}
	
	/// <summary>
	/// The IWMMetadataEditor interface is used to edit metadata information in ASF file headers.
	/// It is obtained by calling the WMCreateEditor function.
	/// </summary>
	[ComImport, Guid( "96406BD9-2B2B-11d3-B36B-00C04F6108FF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMMetadataEditor
	{
		/// <summary>
		/// The Open method opens an ASF file.
		/// </summary>
		/// <param name="fileName">Pointer to a wide-character null-terminated string containing the file name.</param>
		void Open( [In, MarshalAs( UnmanagedType.LPWStr )] string filename );

		/// <summary>
		/// The Close method closes the open file without saving any changes.
		/// </summary>
		void Close();

		/// <summary>
		/// The Flush method closes the open file, saving any changes.
		/// </summary>
		void Flush();
	}

	/// <summary>
	/// The IWMMetadataEditor2 interface provides an improved method for opening files for metadata operations.
	/// This interface is implemented as part of the metadata editor object.
	/// To obtain a pointer to IWMMetadataEditor2, call the QueryInterface method of any other interface in an existing metadata editor object.
	/// </summary>
	[ComImport, Guid( "203CFFE3-2E18-4fdf-B59D-6E71530534CF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMMetadataEditor2 : IWMMetadataEditor
	{
		/// <summary>
		/// The OpenEx method opens a file for use by the metadata editor object.
		/// OpenEx opens ASF files and MP3 files, though the metadata editor has limited capabilities when working with MP3 files.
		/// </summary>
		/// <param name="fileName">Pointer to a wide-character null-terminated string containing the file name.</param>
		/// <param name="desiredAccess">DWORD containing the desired access type. This can be set to GENERIC_READ or GENERIC_WRITE. For read/write access, pass both values combined with a bitwise OR. When using GENERIC_READ, you must also pass a valid sharing mode as dwShareMode. Failure to do so will result in an error.</param>
		/// <param name="shareMode">DWORD containing the sharing mode. This can be one of the values in the following table or a combination of the two using a bitwise OR. A value of zero indicates no sharing. Sharing is not supported when requesting read/write access. If you request read/write access and pass any value other than zero for the share mode, an error is returned.</param>
		void OpenEx( [In, MarshalAs( UnmanagedType.LPWStr )] string filename, [In] uint desiredAccess, [In] uint shareMode );
	}

	/// <summary>
	/// The IWMHeaderInfo interface sets and retrieves information in the header section of an ASF file.
	/// You can manipulate three types of header information by using the methods of this interface: metadata attributes, markers, and script commands.
	/// </summary>
	[ComImport, Guid( "96406BDA-2B2B-11d3-B36B-00C04F6108FF" ), InterfaceType( ComInterfaceType.InterfaceIsIUnknown )]
	interface IWMHeaderInfo
	{
		/// <summary>
		/// The GetAttributeCount method returns the number of attributes defined in the header section of the ASF file.
		/// This method is replaced by IWMHeaderInfo3::GetAttributeCountEx and IWMHeaderInfo3::GetAttributeIndices, and should no longer be used.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number. Pass zero for file-level attributes.</param>
		/// <param name="attributes">Pointer to a count of the attributes.</param>
		void GetAttributeCount( [In] ushort streamNumber, [Out] out ushort attributes );

		/// <summary>
		/// The GetAttributeByIndex method returns a descriptive attribute that is stored in the header section of the ASF file.
		/// This method is replaced by IWMHeaderInfo3::GetAttributeByIndexEx and should not be used.
		/// </summary>
		/// <param name="index">WORD containing the index.</param>
		/// <param name="streamNumber">Pointer to a WORD containing the stream number. Although this parameter is a pointer, the method will not change the value. For file-level attributes, use zero for the stream number.</param>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the name. Pass NULL to this parameter to retrieve the required length for the name. Attribute names are limited to 1024 wide characters.</param>
		/// <param name="nameLength">On input, a pointer to a variable containing the length of the pwszName array in wide characters (2 bytes). On output, if the method succeeds, the variable contains the actual length of the name, including the terminating null character.</param>
		/// <param name="type">Pointer to a variable containing one value from the WMT_ATTR_DATATYPE enumeration type.</param>
		/// <param name="value">Pointer to a byte array containing the value. Pass NULL to this parameter to retrieve the required length for the value.</param>
		/// <param name="valueLength">On input, a pointer to a variable containing the length of the pValue array, in bytes. On output, if the method succeeds, the variable contains the actual number of bytes written to pValue by the method.</param>
		void GetAttributeByIndex( [In] ushort index, [In, Out] ref ushort streamNumber, [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder name, [In, Out] ref ushort nameLength, [Out] out WMT_ATTR_DATATYPE type, [Out, MarshalAs( UnmanagedType.LPArray )] byte[] value, [In, Out] ref ushort valueLength );

		/// <summary>
		/// The GetAttributeByName method returns a descriptive attribute that is stored in the header section of the ASF file.
		/// Now that attribute names can be duplicated in a file, this method is obsolete.
		/// To find attributes of a particular name, use IWMHeaderInfo3::GetAttributeIndices.
		/// </summary>
		/// <param name="streamNumber">Pointer to a WORD containing the stream number, or zero to indicate any stream. Although this parameter is a pointer, the method does not change the value.</param>
		/// <param name="name">Pointer to a null-terminated string containing the name of the attribute. Attribute names are limited to 1024 wide characters.</param>
		/// <param name="type">Pointer to a variable that receives a value from the WMT_ATTR_DATATYPE enumeration type. The returned value specifies the data type of the attribute.</param>
		/// <param name="value">Pointer to a byte array that receives the value of the attribute. The caller must allocate the array. To determine the required array size, set this parameter to NULL and check the value returned in the pcbLength parameter.</param>
		/// <param name="valueLength">On input, the length of the pValue array, in bytes. On output, if the method succeeds, the actual number of bytes that were written to pValue.</param>
		/// <returns>The method returns an HRESULT.</returns>
		[PreserveSig()]
		int GetAttributeByName( [In, Out] ref ushort streamNumber, [In, MarshalAs( UnmanagedType.LPWStr )] string name, [Out] out WMT_ATTR_DATATYPE type, [Out, MarshalAs( UnmanagedType.LPArray )] byte[] value, [In, Out] ref ushort valueLength );

		/// <summary>
		/// The SetAttribute method sets a descriptive attribute that is stored in the header section of the ASF file.
		/// This method is replaced by IWMHeaderInfo3::AddAttribute, and should not be used.
		/// </summary>
		/// <param name="streamNumber">WORD containing the stream number. To set a file-level attribute, pass zero.</param>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the name of the attribute. Attribute names are limited to 1024 wide characters.</param>
		/// <param name="type">A value from the WMT_ATTR_DATATYPE enumeration type.</param>
		/// <param name="value">Pointer to a byte array containing the value of the attribute.</param>
		/// <param name="valueLength">The size of pValue, in bytes.</param>
		void SetAttribute( [In] ushort streamNumber, [In, MarshalAs( UnmanagedType.LPWStr )] string name, [In] WMT_ATTR_DATATYPE type, [In, MarshalAs( UnmanagedType.LPArray )] byte[] value, [In] ushort valueLength );

		/// <summary>
		/// The GetMarkerCount method returns the number of markers currently in the header section of the ASF file.
		/// </summary>
		/// <param name="count">Pointer to a count of markers.</param>
		void GetMarkerCount( [Out] out ushort count );

		/// <summary>
		/// The GetMarker method returns the name and time of a marker.
		/// </summary>
		/// <param name="index">WORD containing the index.</param>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the marker name.</param>
		/// <param name="nameLength">On input, a pointer to a variable containing the length of the pwszMarkerName array in wide characters (2 bytes). On output, if the method succeeds, the variable contains the actual length of the name, including the terminating null character. To retrieve the length of the name, you must set this to zero and set pwszMarkerName and pcnsMarkerTime to NULL.</param>
		/// <param name="time">Pointer to a variable specifying the marker time in 100-nanosecond increments.</param>
		void GetMarker( [In] ushort index, [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder name, [In, Out] ref ushort nameLength, [Out] out ulong time );

		/// <summary>
		/// The AddMarker method adds a marker, consisting of a name and a specific time, to the header section of the ASF file.
		/// </summary>
		/// <param name="name">Pointer to a wide-character null-terminated string containing the marker name. Marker names are limited to 5120 wide characters.</param>
		/// <param name="time">The marker time in 100-nanosecond increments.</param>
		void AddMarker( [In, MarshalAs( UnmanagedType.LPWStr )] string name, [In] ulong time );

		/// <summary>
		/// The RemoveMarker method removes a marker from the header section of the ASF file.
		/// </summary>
		/// <param name="index">WORD containing the index of the marker.</param>
		void RemoveMarker( [In] ushort index );

		/// <summary>
		/// The GetScriptCount method returns the number of scripts currently in the header section of the ASF file.
		/// </summary>
		/// <param name="count">Pointer to a count of scripts.</param>
		void GetScriptCount( [Out] out ushort count );

		/// <summary>
		/// The GetScript method returns the type and command strings, and presentation time of a script.
		/// </summary>
		/// <param name="index">WORD containing the index.</param>
		/// <param name="type">Pointer to a wide-character null-terminated string containing the type.</param>
		/// <param name="typeLength">On input, a pointer to a variable containing the length of the pwszType array in wide characters (2 bytes). On output, if the method succeeds, the variable contains the actual length of the string loaded into pwszType, including the terminating null character. To retrieve the length of the type, you must set this to zero and set pwszType to NULL.</param>
		/// <param name="command">Pointer to a wide-character null-terminated string containing the command.</param>
		/// <param name="commandLength">On input, a pointer to a variable containing the length of the pwszCommand array in wide characters (2 bytes). On output, if the method succeeds, the variable contains the actual length of the command string, including the terminating null character. To retrieve the length of the command, you must set this to zero and set pwszCommand to NULL.</param>
		/// <param name="time">Pointer to a QWORD specifying the presentation time of this script command in 100-nanosecond increments.</param>
		void GetScript( [In] ushort index, [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder type, [In, Out] ref ushort typeLength, [Out, MarshalAs( UnmanagedType.LPWStr )] StringBuilder command, [In, Out] ref ushort commandLength, [Out] out ulong time );

		/// <summary>
		/// The AddScript method adds a script, consisting of type and command strings, and a specific time, to the header section of the ASF file.
		/// </summary>
		/// <param name="type">Pointer to a wide-character null-terminated string containing the type. Script types are limited to 1024 wide characters.</param>
		/// <param name="command">Pointer to a wide-character null-terminated string containing the command. Script commands are limited to 10240 wide characters.</param>
		/// <param name="time">The script time in 100-nanosecond increments.</param>
		void AddScript( [In, MarshalAs( UnmanagedType.LPWStr )] string type, [In, MarshalAs( UnmanagedType.LPWStr )] string command, [In] ulong time );

		/// <summary>
		/// The RemoveScript method enables the object to remove a script from the header section of the ASF file.
		/// </summary>
		/// <param name="index">WORD containing the index of the script.</param>
		void RemoveScript( [In] ushort index );
	}


}
