﻿using System;
using System.Text;

namespace WpfAudioPlayer.Models.Wmf
{
	/// <summary>
	/// IWMHeaderInfo の操作を簡易にする為のユーティリティ クラス。
	/// </summary>
	class WmfHeaderInfo
	{
		/// <summary>
		/// メタデータを操作する為のインターフェース。
		/// </summary>
		private IWMHeaderInfo _wmHeaderInfo;

		/// <summary>
		/// メタデータを操作する為のインターフェースを取得する。
		/// </summary>
		public IWMHeaderInfo HeaderInfo { get { return this._wmHeaderInfo; } }

		/// <summary>
		/// コンストラクタ。
		/// </summary>
		/// <param name="wmHeaderInfo">メタデータを操作する為のインターフェース</param>
		public WmfHeaderInfo( IWMHeaderInfo wmHeaderInfo )
		{
			this._wmHeaderInfo = wmHeaderInfo;
		}

		/// <summary>
		/// Windows Media の属性データをタグ情報に変換する。
		/// </summary>
		/// <param name="value">属性データ。</param>
		/// <param name="type">属性データの型。</param>
		/// <returns>成功時は変換したタグ情報、失敗時は null を返す。</returns>
		private static WmfTagValue ConvertTagValue( byte[] value, WMT_ATTR_DATATYPE type )
		{
			switch( type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING:

				if( value.Length >= 2 )
				{
					StringBuilder sb = new StringBuilder( value.Length / 2 );
					for( int i = 0; i < value.Length - 2; i += 2 )
					{
						sb.Append( BitConverter.ToChar( value, i ) );
					}

					return new WmfTagValue() { Type = type, String = sb.ToString() };
				}
				return null;

			case WMT_ATTR_DATATYPE.WMT_TYPE_BINARY: return new WmfTagValue() { Type = type, Binary = value };
			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return new WmfTagValue() { Type = type, Boolean = BitConverter.ToBoolean( value, 0 ) };
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return new WmfTagValue() { Type = type, DWord = BitConverter.ToUInt32( value, 0 ) };
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return new WmfTagValue() { Type = type, QWord = BitConverter.ToUInt64( value, 0 ) };
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return new WmfTagValue() { Type = type, Word = BitConverter.ToUInt16( value, 0 ) };
			case WMT_ATTR_DATATYPE.WMT_TYPE_GUID: return new WmfTagValue() { Type = type, Guid = new Guid( BitConverter.ToString( value, 0, value.Length ) ) };
			default: return null;
			}
		}

		/// <summary>
		/// タグ情報を取得する。
		/// </summary>
		/// <param name="attributeName">属性名。</param>
		/// <returns>成功時はタグ情報、失敗時は null を返す。</returns>
		public WmfTagValue GetTagValue( string attributeName )
		{
			WMT_ATTR_DATATYPE	dataType;
			byte[]				value        = null;
			ushort				valueLength  = 0;
			ushort				streamNumber = 0xFFFF;

			int ret = this._wmHeaderInfo.GetAttributeByName( ref streamNumber, attributeName, out dataType, value, ref valueLength );
			if( ret != NativeMethods.S_OK ) { return null; }

			value = new byte[ valueLength ];
			ret = this._wmHeaderInfo.GetAttributeByName( ref streamNumber, attributeName, out dataType, value, ref valueLength );
			if( ret != NativeMethods.S_OK ) { return null; }

			return WmfHeaderInfo.ConvertTagValue( value, dataType );
		}
	}

	/// <summary>
	/// 一つのタグ情報を格納する為のクラス。
	/// </summary>
	class WmfTagValue
	{
		/// <summary>
		/// データの種別を設定または取得する。
		/// </summary>
		public WMT_ATTR_DATATYPE Type { get; set; }

		/// <summary>
		/// 文字列データを取得または設定する。
		/// </summary>
		public string String { get; set; }

		/// <summary>
		/// バイト配列を取得または設定する。
		/// </summary>
		public byte[] Binary { get; set; }

		/// <summary>
		/// 真偽値を取得または設定する。
		/// </summary>
		public bool Boolean { get; set; }

		/// <summary>
		/// 4 バイトの整数を取得または設定する。
		/// </summary>
		public uint DWord { get; set; }

		/// <summary>
		/// 8 バイトの整数を取得または設定する。
		/// </summary>
		public ulong QWord { get; set; }

		/// <summary>
		/// 2 バイトの整数を取得または設定する。
		/// </summary>
		public ushort Word { get; set; }

		/// <summary>
		/// 識別子を取得または設定する。
		/// </summary>
		public Guid Guid { get; set; }

		/// <summary>
		/// 現在の値を日時に変換する。
		/// </summary>
		/// <returns>変換した日時を返す。</returns>
		public DateTime ToDateTime()
		{
			if( this.Type == WMT_ATTR_DATATYPE.WMT_TYPE_STRING )
			{
				DateTime dateTime;
				if( DateTime.TryParse( this.String, out dateTime ) )
				{
					return dateTime;
				}
				else
				{
					int year;
					if( int.TryParse( this.String, out year ) )
					{
						dateTime.AddYears( year );
						return dateTime;
					}
				}
			}

			return new DateTime();
		}

		/// <summary>
		/// 現在の値を文字列に変換する。
		/// </summary>
		/// <returns>変換した文字列を返す。</returns>
		public override string ToString()
		{
			switch( this.Type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING: return this.String;
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return this.Word.ToString();
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return this.DWord.ToString();
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return this.QWord.ToString();
			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return this.Boolean.ToString();
			case WMT_ATTR_DATATYPE.WMT_TYPE_GUID: return this.Guid.ToString();
			default: return String.Empty;
			}
		}

		/// <summary>
		/// 現在の値を真偽値に変換する。
		/// </summary>
		/// <returns>変換した真偽値を返す。</returns>
		public bool ToBoolean()
		{
			switch( this.Type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING:
				{
					bool flag = false;
					bool.TryParse( this.String, out flag );
					return flag;
				}

			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return this.Boolean;
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return ( this.Word == 0 ? false : true );
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return ( this.DWord == 0 ? false : true );
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return ( this.QWord == 0 ? false : true );
			default: return false;
			}
		}

		/// <summary>
		/// 現在の値を 2 バイト ( 16 ビット ) の整数値に変換する。
		/// </summary>
		/// <returns>変換した 2 バイト ( 16 ビット ) の整数値を返す。</returns>
		public ushort ToWord()
		{
			switch( this.Type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING:
				{
					ushort value = 0;
					ushort.TryParse( this.String, out value );
					return value;
				}

			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return ( ushort )( this.Boolean ? 1 : 0 );
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return this.Word;
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return ( ushort )this.DWord;
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return ( ushort )this.QWord;
			default: return 0;
			}
		}

		/// <summary>
		/// 現在の値を 4 バイト ( 32 ビット ) の整数値に変換する。
		/// </summary>
		/// <returns>変換した 4 バイト ( 32 ビット ) の整数値を返す。</returns>
		public uint ToDWord()
		{
			switch( this.Type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING:
				{
					uint value = 0;
					uint.TryParse( this.String, out value );
					return value;
				}

			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return ( this.Boolean ? 1U : 0 );
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return this.Word;
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return this.DWord;
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return ( uint )this.QWord;
			default: return 0;
			}
		}

		/// <summary>
		/// 現在の値を 8 バイト ( 64 ビット ) の整数値に変換する。
		/// </summary>
		/// <returns>変換した 8 バイト ( 64 ビット ) の整数値を返す。</returns>
		public ulong ToQWord()
		{
			switch( this.Type )
			{
			case WMT_ATTR_DATATYPE.WMT_TYPE_STRING:
				{
					ulong value = 0;
					ulong.TryParse( this.String, out value );
					return value;
				}

			case WMT_ATTR_DATATYPE.WMT_TYPE_BOOL: return ( this.Boolean ? 1U : 0 );
			case WMT_ATTR_DATATYPE.WMT_TYPE_WORD: return this.Word;
			case WMT_ATTR_DATATYPE.WMT_TYPE_DWORD: return this.DWord;
			case WMT_ATTR_DATATYPE.WMT_TYPE_QWORD: return this.QWord;
			default: return 0;
			}
		}
	}
}
